package com.duanledexianxianxian.maven.flink.common.util;

import com.duanledexianxianxian.maven.flink.common.constant.AppConstants;
import org.apache.flink.api.common.restartstrategy.RestartStrategies;
import org.apache.flink.api.java.utils.ParameterTool;
import org.apache.flink.streaming.api.TimeCharacteristic;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * 执行环境参数工具类
 *
 * @author Administrator
 */
public class ExecutionEnvUtils {

    /**
     * Create parameter tool parameter tool.
     *
     * @param args the args
     * @return the parameter tool
     * @throws Exception the exception
     */
    public static ParameterTool createParameterTool(final String[] args) throws Exception {
        return ParameterTool
                .fromPropertiesFile(ExecutionEnvUtils.class.getResourceAsStream(AppConstants.PROPERTIES_FILE_NAME))
                .mergeWith(ParameterTool.fromArgs(args))
                .mergeWith(ParameterTool.fromSystemProperties())
                .mergeWith(ParameterTool.fromMap(getenv()));
    }


    /**
     * The constant PARAMETER_TOOL.
     */
    public static final ParameterTool PARAMETER_TOOL = createParameterTool();

    private static ParameterTool createParameterTool() {
        try {
            return ParameterTool
                    .fromPropertiesFile(ExecutionEnvUtils.class.getResourceAsStream(AppConstants.PROPERTIES_FILE_NAME))
                    .mergeWith(ParameterTool.fromSystemProperties())
                    .mergeWith(ParameterTool.fromMap(getenv()));
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 将环境变量中的值转换成map结构
     *
     * @return
     */
    private static Map<String, String> getenv() {
        Map<String, String> map = new HashMap<>();
        for (Map.Entry<String, String> entry : System.getenv().entrySet()) {
            map.put(entry.getKey(), entry.getValue());
        }
        return map;
    }


    /**
     * 设置默认参数
     *
     * @param parameterTool the parameter tool
     * @return the stream execution environment
     * @throws Exception the exception
     */
    public static StreamExecutionEnvironment prepare(ParameterTool parameterTool) throws Exception {
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        // 设置作业的默认并行度
        env.setParallelism(parameterTool.getInt(AppConstants.STREAM_PARALLELISM, 5));
        // 关闭系统日志
        env.getConfig().disableSysoutLogging();
        // 失败之后，重拾4次，每次间隔10秒
        env.getConfig().setRestartStrategy(RestartStrategies.fixedDelayRestart(4, 10000));
        if (parameterTool.getBoolean(AppConstants.STREAM_CHECKPOINT_ENABLE, true)) {
            // create a checkpoint every 5 seconds
            env.enableCheckpointing(parameterTool.getInt(AppConstants.STREAM_CHECKPOINT_INTERVAL, 1000));
        }
        // make parameters available in the web interface
        env.getConfig().setGlobalJobParameters(parameterTool);
        // 默认是事件时间
        env.setStreamTimeCharacteristic(TimeCharacteristic.EventTime);
        return env;
    }
}

