package com.duanledexianxianxian.maven.flink.sql.sql;


import com.google.common.collect.Lists;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.flink.streaming.api.TimeCharacteristic;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.timestamps.BoundedOutOfOrdernessTimestampExtractor;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.table.api.Table;
import org.apache.flink.table.api.java.StreamTableEnvironment;
import org.apache.flink.types.Row;

import java.util.List;

/**
 * The type Main 1.
 *
 * @author fengyuchenglun
 * @version 1.0.0
 */
public class PageAccessSql {
    /**
     * The constant OUT_OF_ORDERNESS.
     */
    public static final int OUT_OF_ORDERNESS = 1000;

    private static final Object[][] INIT_DATA = {
            {"ShangHai", "U0010", 1510365660000L},
            {"BeiJing", "U1001", 1510365660000L},
            {"BeiJing", "U2032", 1510366200000L},
            {"BeiJing", "U1100", 1510366260000L},
            {"ShangHai", "U0011", 1510373400000L},
    };

    /**
     * The entry point of application.
     *
     * @param args the input arguments
     * @throws Exception the exception
     */
    public static void main(String[] args) throws Exception {
        // Streaming 环境
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        //EnvironmentSettings settings = EnvironmentSettings.newInstance()
        //        .useBlinkPlanner()
        //        .inStreamingMode()
        //        .build();

        //StreamTableEnvironment tableEnv = StreamTableEnvironment.create(env, settings);

        StreamTableEnvironment tableEnv = StreamTableEnvironment.create(env);

        // 设置EventTime
        env.setStreamTimeCharacteristic(TimeCharacteristic.EventTime);

        //方便我们查出输出数据
        env.setParallelism(1);

        // 初始化数据
        List<PageAccess> pageAccessList = Lists.newArrayList();
        initData(pageAccessList);
        // 设置水印
        DataStream<PageAccess> stream = env.fromCollection(pageAccessList).assignTimestampsAndWatermarks(new TimestampsAndWatermarks());


//        Table pageAccess = tableEnv.fromDataStream(stream, "region,userId,accessTime");
        // register DataStream as Table
        tableEnv.registerDataStream("pageAccess", stream, "region,userId,accessTime,rowTime.rowtime");
        // oder by 主排序必须是时间属性字段
        Table result = tableEnv.sqlQuery("select * from pageAccess  order by rowTime ,region  asc");


        // table=>DataStream
        DataStream<Row> rowDataStream = tableEnv.toAppendStream(result, Row.class);

        rowDataStream.print();

        // execute
        env.execute("Flink sql");

    }

    /**
     * Init data.
     *
     * @param pageAccessList the page access list
     */
    public static void initData(List<PageAccess> pageAccessList) {
        for (Object[] initDatum : INIT_DATA) {
            PageAccess pageAccess = new PageAccess(String.valueOf(initDatum[0]), String.valueOf(initDatum[1]), Long.valueOf(String.valueOf(initDatum[2])));
            pageAccessList.add(pageAccess);
        }
    }

    /**
     * The type Page access.
     *
     * @author fengyuchenglun
     * @version 1.0.0
     */
    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public static class PageAccess {
        // 区域
        private String region;
        // 用户名称
        private String userId;
        // 访问时间
        private Long accessTime;
    }

    /**
     * 时间水印
     * BoundedOutOfOrdernessTimestampExtractor处理乱序
     */
    private static class TimestampsAndWatermarks extends BoundedOutOfOrdernessTimestampExtractor<PageAccess> {

        /**
         * Instantiates a new Timestamps and watermarks.
         */
        public TimestampsAndWatermarks() {
            super(Time.milliseconds(0));
        }

        @Override
        public long extractTimestamp(PageAccess pageAccess) {
            // 提取eventTime
            return pageAccess.getAccessTime();
        }
    }
}
